package cup.chap3.db;
import java.sql.*;
import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class DBServlet extends HttpServlet implements SingleThreadModel {
	private Connection con=null;

	public void init(ServletConfig conf) throws ServletException {
      	super.init(conf);
		try {
			Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");  
			String url = "jdbc:odbc:bugdb";
			con =DriverManager.getConnection(url,"dba","sql");
		} catch (Exception e) {
			System.out.println(e);
		}
	}

	public void doGet(HttpServletRequest req, HttpServletResponse res) 
        throws ServletException, IOException {
		PrintWriter	out;
		res.setContentType("text/html");
		out = res.getWriter();
		displayHeader(out);	
		displayQueryInterface(out);
		displayFooter(out);	
	}

	public void doPost(HttpServletRequest req, HttpServletResponse res) 
        throws ServletException, IOException {
		PrintWriter	out;
		res.setContentType("text/html");
		out = res.getWriter();
		String query = req.getParameter("query");
		try {
			Statement stmt = con.createStatement();
			ResultSet rs = stmt.executeQuery(query);
			displayHeader(out);	
			displayQueryInterface(out);
			out.println("<TABLE BORDER CELLPADDING=8 CELLSPACING=4>");
			displayResultSet(rs, out);
			out.println("</TABLE>");
			displayFooter(out);
			rs.close();
		} catch (Exception e) {
			System.out.println(e);
		}
	}

	private void displayHeader(PrintWriter out) {
		out.println("<HTML>");
		out.println("<HEAD><TITLE>General Purpose Query Processor</TITLE></HEAD>");
		out.println("<BODY>");
		out.println("<CENTER>");
	}

	private void displayQueryInterface(PrintWriter out) {
		out.println("<FORM METHOD=POST ACTION=http://localhost:8181/servlet/dbservlet>");
		out.println("Please Input Your Query: <INPUT NAME=query SIZE=40>");
		out.println("<INPUT TYPE=SUBMIT VALUE='Submit Query'>");
		out.println("</FORM>");
	}

	private void displayFooter(PrintWriter out) {
		out.println("</CENTER>");
		out.println("</BODY>");
		out.println("</HTML>");
		out.close();
	}

	private void displayResultSet(ResultSet rs, PrintWriter out) throws SQLException {
		ResultSetMetaData metaData = rs.getMetaData();
		int cols =metaData.getColumnCount();	
		out.println("<TR>");
		for(int i=1;i<=cols;++i) {
			out.println("<TH> "+metaData.getColumnName(i)+" </TH>");
		}
		out.println("</TR>");
	
		while (rs.next()) {
			out.println("<TR>");
	  		for (int i=1; i<=cols; i++) {
				displayColumnData(rs, out, metaData.getColumnType(i), i);
	      	}
			out.println("</TR>");
	      }
	}

	private void displayColumnData(ResultSet rs, PrintWriter out, int columnType, int col)
      	throws SQLException  {

		switch(columnType) {
			case Types.CHAR:
			case Types.VARCHAR:
			case Types.LONGVARCHAR:
				String s = rs.getString(col);
				out.println("<TD>" + s + "</TD>");
				break;
			case Types.NUMERIC:
			case Types.DECIMAL:
				java.math.BigDecimal num = rs.getBigDecimal(col, 10);
				out.println("<TD>" + num.toString() + "</TD>");
				break;
			case Types.INTEGER:
				int i = rs.getInt(col);
				out.println("<TD>" + new Integer(i) + "</TD>");
				break;
			case Types.BIGINT:
				long bigi = rs.getLong(col);
				out.println("<TD>" + new Long(bigi) + "</TD>");
				break;
			case Types.SMALLINT:
				short smalli = rs.getShort(col);
				out.println("<TD>" + new Integer(smalli) + "</TD>");
				break;
			case Types.TINYINT:
				byte tinyi = rs.getByte(col);
				out.println("<TD>" + new Integer(tinyi) + "</TD>");
				break;
			case Types.REAL:
				float real = rs.getFloat(col);
				out.println("<TD>" + new Float(real) + "</TD>");
				break;
			case Types.FLOAT:
			case Types.DOUBLE:
				double longreal = rs.getDouble(col);
				out.println("<TD>" + new Double(longreal) + "</TD>");
				break;
			case Types.BIT:
				boolean bit = rs.getBoolean(col);
				out.println("<TD>" + new Boolean(bit) + "</TD>");
				break;
			case Types.BINARY:
			case Types.VARBINARY:
			case Types.LONGVARBINARY:
				byte[] longbin = rs.getBytes(col);
				out.println("<TD>" + new String(longbin, 0) + "</TD>");
				break;
			case Types.DATE:
				java.sql.Date date = rs.getDate(col);
				out.println("<TD>" + date.toString() + "</TD>");
				break;
			case Types.TIME:
				java.sql.Time time = rs.getTime(col);
				out.println("<TD>" + time.toString() + "</TD>");
				break;
			case Types.TIMESTAMP:
				java.sql.Timestamp ts = rs.getTimestamp(col);
				out.println("<TD>"+ ts.toString() + "</th>");
				break;
		}
	}

	public void destroy() {
		try {
			con.close();
		} catch (Exception e) {
			System.out.println(e);
		}
	}
}
