/**
	CallJava.c
	Using the JNIs  Invocation API, it is 
possible for C/C++ program to seamlessly 
	load the JVM and execute Java classes within it. 
This program demonstrates how a C program can 
instantiate a JVM, load the HelloC class, create 
an object, and invoke its methods.
*/


#include <jni.h>

#ifdef _WIN32
#define PATH_SEPARATOR ';'
#else /* UNIX */
#define PATH_SEPARATOR ':'
#endif

/* set location of our test java class HelloC  here*/
#define USER_CLASSPATH "." 

main() {
    JNIEnv *env;
    JavaVM *jvm;
    JDK1_1InitArgs vm_args;
    jint result;
    jclass cls;
    jobject jo;
    jmethodID ctorCls;
    jmethodID mid;
    char classpath[1000];


    /* indicate version number if you use 
  JDK1.1.2 and beyond */

    vm_args.version = 0x00010001;

    JNI_GetDefaultJavaVMInitArgs(&vm_args);

    /* Append local classpath to the end of 
  default system classpath */
    
sprintf(classpath, "%s%c%s",
vm_args.classpath, PATH_SEPARATOR, USER_CLASSPATH);
vm_args.classpath = classpath;

    /* Create the Java VM */
    result = JNI_CreateJavaVM(&jvm,&env,&vm_args);
    if (result < 0) {
        fprintf(stderr, "Can't create Java VM\n");
        exit(1);
    }

    /* load the test class */
    cls = (*env)->FindClass(env, "HelloC");
    if (cls == 0) {
        fprintf(stderr, "Can't find class HelloC\n");
        exit(1);
    }

    /* instantiate an object  of this class*/
    ctorCls = (jmethodID) 	(*env)->GetMethodID (env,cls,"<init>","()V");
    jo =  (*env)->NewObject(env, cls,ctorCls);
 
    /* invoke the method */
    mid = (*env)->GetMethodID(env, cls, "sayHello", "()V");
    if (mid == 0) {
        fprintf(stderr, "Can't find HelloC.sayHello\n");
        exit(1);
    }
    (*env)->CallVoidMethod(env, jo, mid);

    (*jvm)->DestroyJavaVM(jvm);
}
