import net.jini.core.entry.*; 
import net.jini.core.lookup.*; 
import net.jini.core.discovery.*; 
import net.jini.lookup.*; 
import net.jini.lookup.entry.*; 
import net.jini.discovery.*; 
import java.rmi.*; 
/*************************************************************************** 

 EchoClient -- The Jini Client Class 

 ***************************************************************************/ 

class EchoClient { 
 
 public static void main (String[] args) { 
  
  int iPort; 
  String sHost; 
  Entry[] aeAttributes; 
  LookupLocator lookup; 
  ServiceID id; 
  ServiceRegistrar registrar; 
  ServiceTemplate template; 
  EchoInterface myServerInterface; 

  try { 
   /* 
   Setting the security manager to allow the RMI class loader 
   to go to the codebase for classes that are not available 
   locally. 
   ========================================================== */ 
   System.setSecurityManager (new RMISecurityManager ()); 
   

   /* 
   Find the Jini lookup service (reggie) and print its location. 
   ============================================================= */ 
   lookup = new LookupLocator ("jini://" + args[0]); 
   sHost = lookup.getHost (); 
   iPort = lookup.getPort (); 
   System.out.println (); 
   System.out.println ("client: LookupLocator = " + lookup); 
   System.out.println ("client: LookupLocator.host = " + sHost); 
   System.out.println ("client: LookupLocator.port = " + iPort); 
   

   /* 
   Get the lookup service's ServiceRegistrar (the class by which 
   interaction with the lookup service is possible). 
   ============================================================= */ 
   registrar = lookup.getRegistrar (); 
   id = registrar.getServiceID (); 
   System.out.println ("client: ServiceRegistrar = " + registrar); 
   System.out.println ("client: ServiceID = " + id); 
   

   /* 
   Perform a search on the lookup server to find the service 
   that has the name attribute of "EchoServer". The lookup 
   service returns an interface object to the service. 
   ========================================================= */ 
   aeAttributes = new Entry[1]; 
   aeAttributes[0] = new Name ("EchoServer"); 
   template = new ServiceTemplate (null, null, aeAttributes); 
   myServerInterface = (EchoInterface) registrar.lookup (template); 
   System.out.println ("client: ServiceTemplate = " + template); 
   System.out.println ("client: Service object = " + myServerInterface); 
   

   /* 
   If the correct object was returned, call one of its methods. 
   ============================================================ */ 
   if (myServerInterface instanceof EchoInterface) { 
    long startTime = System.currentTimeMillis();  
    System.out.println ("============================================================"); 
    System.out.println ("client: Time at server is " + startTime); 
    String message = myServerInterface.echoMessage (args[1]);
    long stopTime = System.currentTimeMillis(); 
    System.out.println ("client: Time at client is " + stopTime); 
    float difference= stopTime-startTime; 
    System.out.println ("client: The Server echoed thus --->" 
                        + message); 
    System.out.println ("client: Time for message to traverse the net is --->" 
                        + difference + " msecs<---"); 
    System.out.println ("============================================================"); 
   } 
  } catch (Exception e) { System.out.println ("client: EchoClient.main() exception: " + e); } 
 } 
} 

