
import net.jini.core.entry.*;
import net.jini.core.lookup.*;
import net.jini.core.discovery.*; 
import net.jini.lookup.entry.*;
import net.jini.discovery.*;
import com.sun.jini.lookup.*;
import com.sun.jini.lease.*;

import java.io.*;
import java.rmi.*;
import java.rmi.server.*;
/***************************************************************************

 EchoServer -- The Jini Server Class

 ***************************************************************************/

public class EchoServer extends UnicastRemoteObject 
 implements EchoInterface, ServiceIDListener, Serializable {

 public EchoServer() throws RemoteException { 
  super (); 
 }

 // EchoInterface
 public String echoMessage (String message) throws RemoteException { 
  
  System.out.println ("============================================================"); 
  System.out.println ("EchoServer: The method echoMessage() in EchoServer was called");
  System.out.println ("EchoServer: Client says :" + message);
  System.out.println ("============================================================"); 
  
  return (message);
 }

 // ServiceIDListener
 public void serviceIDNotify (ServiceID idIn) { 
 } 


 public static void main (String[] args) {
  
  int iPort;
  String sHost;
  EchoServer myServer;
  LookupLocator lookup;
  Entry [] aeAttributes;
  JoinManager joinmanager;
  ServiceID id; 
  ServiceMatches matches;
  ServiceRegistrar registrar;

  try {
   /* 
   Setting the security manager to allow the RMI class loader
   to go to the codebase for classes that are not available
   locally.
   ========================================================== */
   System.setSecurityManager (new RMISecurityManager ());

   /* 
   Create the attributes (an array of entry objects) that describe 
   this server and use it to register this server with the lookup
   service. JoinManager finds and registers with the lookup 
   service.
   =============================================================== */
   aeAttributes = new Entry [1];
   aeAttributes[0] = new Name ("EchoServer");
   myServer = new EchoServer ();
   joinmanager = new JoinManager (myServer,aeAttributes,myServer,new LeaseRenewalManager());
   System.out.println ("EchoServer: JoinManager = " + joinmanager);

   /* 
   Find the Jini lookup service (reggie) and print its location.
   ============================================================= */
   lookup = new LookupLocator ("jini://" + args[0]);
   sHost = lookup.getHost ();
   iPort = lookup.getPort ();
   System.out.println ("EchoServer: LookupLocator = " + lookup);
   System.out.println ("EchoServer: LookupLocator.host = " + sHost);
   System.out.println ("EchoServer: LookupLocator.port = " + iPort);


   /* 
   Get the lookup service's ServiceRegistrar (the class by which
   interaction with the lookup service is possible).
   ============================================================= */
   registrar = lookup.getRegistrar();
   id = registrar.getServiceID ();
   System.out.println ("EchoServer: ServiceRegistrar = " + registrar);
   System.out.println ("EchoServer: ServiceID = " + id);


   /* 
   Itemize the service items and the service objects that are 
   available in the lookup service. Note: Sometimes this is 
   executed too quickly for the service that was registered above 
   to show up. 
   ============================================================== */
   try {Thread.sleep (2000);} catch (Exception e) {}

   matches = registrar.lookup(new ServiceTemplate (null, null, null),50);
   System.out.println ("EchoServer: ServiceMatches = " + matches);
   System.out.println ("EchoServer: num matches = "+ matches.totalMatches);
   for (int i = 0; i < matches.totalMatches; i++) {
    System.out.println ("EchoServer: svc item " + i + ": " + matches.items[i]);
    System.out.println ("EchoServer: svc object " + i + ": "+ matches.items[i].service);
   }
   System.out.println ("*-----------------------------------------*");
   
  } catch (Exception e) { 
   System.out.println ("EchoServer: EchoServer.main(): Exception "); 
   e.printStackTrace ();
  }
  
 }
}  

