package cup.chap6.activation;

import java.rmi.*;
import java.rmi.activation.*;
import java.io.*;

public class CountServer extends Activatable implements CountInterface {

    private Integer count;
    private File storage;

    public CountServer(ActivationID id, MarshalledObject data) 
	throws RemoteException, ClassNotFoundException, IOException {

	super(id, 0);

        // The MarshalledObject is used to pass any previously
 // persisted state information 
	storage = (File) data.get();

	if (storage.exists()) {
	    // If MarshalledObject is a serialized file, then use the
	    // state info within that to initialize the object
	    this.restoreState();
	} else {
	    // Remote object is starting for the first time
	    count = new Integer(0);
	}
    }

    public Integer getCount() throws RemoteException {

	this.saveState();
	return count;
    }

 
private void restoreState() throws IOException, ClassNotFoundException {
	File f = storage;
	try {
	FileInputStream fis = new FileInputStream(f);
        ObjectInputStream ois = new ObjectInputStream(fis);
        count = (Integer)ois.readObject();
        ois.close();
	} catch (Exception e) {
		System.out.println("Exception: "+e);
	}
 }

    private synchronized void saveState() {

        FileOutputStream fos = null;
        ObjectOutputStream oos = null;

	    File f  = storage;
		try {
		     fos = new FileOutputStream(f);
		     oos = new ObjectOutputStream(fos);
			count = new Integer(count.intValue()+1);
	    		oos.writeObject(count);
		    	oos.close();
		} catch (IOException e) {
	        System.out.println("Exception: "+e);
		}
    }
}
