package cup.chap6.rmicallback;

import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.registry.LocateRegistry;
import java.util.*;

public class QuoteServer extends UnicastRemoteObject implements QServerInterface, Runnable {
	private Vector clientele; //tracks all connected clients
	private QClientInterface myQClientInterface; 
	private Vector quoteList; //maintains updatable list of quotes
	private QClientInterface myClientObj;
	private Thread clientThread=null;
	private static int counter=0;

	public QuoteServer ()  throws java.rmi.RemoteException {
		super();
		// create and initialize the quote list 
		quoteList = new Vector();
		quoteList.addElement("Climb mountains to see lowlands");
quoteList.addElement("If you want the rainbow, put up with the rain");
quoteList.addElement("Smooth seas do not make skilful sailors");

		clientele = new Vector();
	}
	
	public  void setQClientInterface(QClientInterface c) throws RemoteException {
		synchronized (clientele) {
			clientele.addElement(c);
		}
		if (clientThread == null) {
			clientThread = new Thread(this, "clientThread");
			clientThread.start();
		}
	}

	private void doIt() {
		synchronized (clientele) {
			Vector backup = (Vector) clientele.clone();
			int seed;
			while ((seed = new Random().nextInt()) <=0) ;
				seed = seed % quoteList.size();
				String data = quoteList.elementAt(seed)+ ": "+ ++counter;

				for (int i=0; i < clientele.size(); i++) {
					myClientObj = (QClientInterface) clientele.elementAt(i);
					try {
						//update the client asynchronously via callback
						myClientObj.refreshClient((String) data );
					} catch (RemoteException e) {
						System.out.println("client must have disconnected!");
						//get rid of the remote reference for disconnected client
						backup.removeElement(myClientObj);
						if (backup.size() <= 0) {
							//no more clients- so stop server thread
							clientele = (Vector) backup.clone();
							Thread dummy = clientThread;
							clientThread = null;
							dummy.stop();
						}
					}
			}
			clientele = (Vector) backup.clone();
		} //end syncronization on clientele
	}

	public void run() {
		while (true) {
			try {
				//sleep for a second
				Thread.currentThread().sleep(1000);
			} catch (Exception e) { 
                                    }
			doIt();
		}
	}	
	
	public void addQuote(String quote) throws RemoteException {
		synchronized (quoteList) {
			//update quote list
			quoteList.addElement(quote);
		}
	}

	public static void main(String[] args) {
		System.setSecurityManager(new RMISecurityManager());
		try {
			System.out.println("QuoteServer.main: creating registry");	
			LocateRegistry.createRegistry(1099);
			System.out.println("QuoteServer.main: creating server");
		            QuoteServer myQuoteServer=new QuoteServer();
		           System.out.println("QuoteServer.main: binding server ");
	                       Naming.rebind("/QuoteServer", myQuoteServer);
	     	           System.out.println("QuoteServer bound in registry");
		 } catch (Exception e) {
			System.out.println("Exception on binding QuoteServer");
			System.out.println(e.toString());
	      }
	}
}
