

package cup.chap8.Shop.Cart;

import java.rmi.*;
import javax.ejb.*;
import java.util.*;
import cup.chap8.Shop.Books.*;
import cup.chap8.Shop.Music.*;
import cup.chap8.Shop.Cart.ReturnSet;

public class CartBean implements SessionBean { 
  
  SessionContext		sessionContext;
  
  public BooksHome	booksHome;
  public MusicHome	musicHome;
  
  public Vector		booksList = null, booksCart = null;
  public Vector		musicList = null, musicCart = null;
  int	   totalBooks = 0, totalMusic = 0;
  double booksCost = 0, musicCost = 0, grandTotal = 0;
  
  void init () { 
    booksList = new Vector ();
    booksCart = new Vector ();
    musicList = new Vector ();
    musicCart = new Vector ();
    ReturnSet result = null;
    for (int i=0; ;i++) { 
      try{
        BooksPK key = new BooksPK (i+1);
        Books found = booksHome.findByPrimaryKey (key);
        if (null == found) {
          break;
        }
        result = new ReturnSet (key.code, found.getTitle (), 
                                found.getAuthors (),
                                found.getPrice (), 
                                found.getDiscount (), 
                                found.getDiscountedPrice ());
        System.out.println ("isbn = "+ result.code +
                            " Title = "+result.title+
                            " Authors = "+result.authors+
                            " Price = " + result.price);
        booksList.addElement (result);

      } catch (Exception e) { e.printStackTrace(); break; }

    }
    for (int i=0; ;i++) { 
      try {
        MusicPK key = new MusicPK (i+1);
        Music found = musicHome.findByPrimaryKey (key);
        if (found == null) {
          break;
        }
        result = new ReturnSet (key.code, found.getAlbum (), 
                                found.getComposers (),
                                found.getPrice (), 
                                found.getDiscount (),
                                found.getDiscountedPrice ());
        System.out.println ("code = "+ result.code +
                            " Album = "+result.title+
                            " Composers = "+result.authors+
                            " Price = " + result.price);
        musicList.addElement (result);

      } catch (Exception e) { e.printStackTrace (); break; }

    }
  }
  
  public Books createNewBook (int isbn, String title, 
                              String authors, 
                              double price, int discount) 
    throws RemoteException, CreateException { 
    
    Books book = booksHome.create (isbn, title, authors, 
                                   price, discount); 
    if (null != book) { 
      booksList.addElement (new ReturnSet (isbn, title, 
                                           authors, 
                                           price, discount, 
                                           book.getDiscountedPrice ())); 
    }
    return book;
  }

  public Music createNewAlbum(int code, String album, String authors, 
                              double price, int discount) 
    throws RemoteException, CreateException { 
    
    Music music = musicHome.create (code, album, authors, 
                                    price, discount); 
    if (null != music) { 
      musicList.addElement (new ReturnSet (code, album, authors, 
                                           price, discount, 
                                           music.getDiscountedPrice ())); 
    }
    return music;
  }
  
  public void addToBooksCart(int isbn) throws RemoteException { 
    
    try { 
      Books book = getBook (isbn);
      if (null != book) { 
        booksCart.addElement (new ReturnSet (isbn, book.getTitle (), 
                                             book.getAuthors (),
                                             book.getPrice (), 
                                             book.getDiscount (),
                                             book.getDiscountedPrice ())); 
        ++totalBooks; 
        booksCost+=book.getDiscountedPrice (); 
        grandTotal+=book.getDiscountedPrice ();
      }
    } catch (Exception e) { e.printStackTrace (); }
  }
  
  public void flushBooksCart () throws RemoteException {
    
    grandTotal -= booksCost;
    totalBooks = 0;
    booksCost = 0;
    booksCart.removeAllElements ();
  }
  
  public void flushMusicCart () throws RemoteException {
    
    grandTotal -= musicCost;
    totalMusic = 0;
    musicCost = 0;
    musicCart.removeAllElements ();
  }
  
  public void addToMusicCart (int code) throws RemoteException { 
    
    try { 
      Music music = getAlbum (code);
      if(null != music) { 
        musicCart.addElement (new ReturnSet (code, music.getAlbum (), 
                                             music.getComposers (),
                                             music.getPrice (), 
                                             music.getDiscount (),
                                             music.getDiscountedPrice ())); 
        ++totalMusic; 
        musicCost+=music.getDiscountedPrice (); 
        grandTotal+=music.getDiscountedPrice ();
      }
    } catch (Exception e) { e.printStackTrace(); }
  }
  
  public Books getBook (int isbn) 
    throws RemoteException, FinderException { 
    
    return booksHome.findByPrimaryKey (new BooksPK (isbn)); 
  }
  
  public Music getAlbum (int code) 
    throws RemoteException, FinderException { 
    
    return musicHome.findByPrimaryKey (new MusicPK (code)); 
  }
  
  public Vector getBooksList () throws RemoteException { 
    return booksList; 
  }
  
  public Vector getMusicList () throws RemoteException { 
    return musicList; 
  }
  
  public Vector getBooksCart () throws RemoteException { 
    return booksCart; 
  }
  
  public Vector getMusicCart () throws RemoteException { 
    return musicCart; 
  }

  public int	  getNumberBooks () throws RemoteException {
    return totalBooks;
  }
  
  public int	  getNumberAlbums () throws RemoteException {
    return totalMusic;
  }
  
  public double getBooksTotal () throws RemoteException {
    return booksCost;
  }
  
  public double getAlbumsTotal () throws RemoteException {
    return musicCost;
  }
  
  public double getGrandTotal () throws RemoteException {
    return grandTotal;
  }
  

  public void ejbCreate () throws RemoteException, CreateException {}
  public void ejbActivate () throws RemoteException {}
  public void ejbPassivate () throws RemoteException {}
  public void ejbRemove () throws RemoteException {}

  public void setSessionContext (SessionContext context) 
    throws RemoteException { 
    
    sessionContext = context; 
    try {
      booksHome = (BooksHome)Naming.lookup ("Books");
      musicHome = (MusicHome)Naming.lookup ("Music");
      init();
    } catch (Exception ex) {
      throw new RemoteException ("Shop not found", ex);
    }
  }
}
